/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.common.utils.excel;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.*;

import javax.servlet.http.HttpServletResponse;

import com.jeeplus.common.utils.SpringContextHolder;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.modules.exampleexpend.service.ExampleExpendService;
import com.jeeplus.modules.sys.entity.Dict;
import com.jeeplus.modules.sys.entity.MainDict;
import com.jeeplus.modules.sys.entity.MainDictDetail;
import com.jeeplus.modules.workexample.entity.WorkExampleIndices;
import com.jeeplus.modules.workjobgrade.entity.WorkJobGrade;
import com.jeeplus.modules.workjobgrade.service.WorkJobGradeService;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.DVConstraint;
import org.apache.poi.hssf.usermodel.HSSFCellStyle;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.ss.util.CellRangeAddressList;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.apache.poi.xssf.usermodel.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.Encodes;
import com.jeeplus.common.utils.Reflections;
import com.jeeplus.common.utils.excel.annotation.ExcelField;
import com.jeeplus.modules.sys.utils.DictUtils;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * 导出Excel文件（导出“XLSX”格式，支持大数据量导出   @see org.apache.poi.ss.SpreadsheetVersion）
 * @author jeeplus
 * @version 2013-04-21
 */
public class ExportExcel {

	private static Logger log = LoggerFactory.getLogger(ExportExcel.class);

    /**
	 * 工作薄对象
	 */
	private SXSSFWorkbook wb;
	
	/**
	 * 工作表对象
	 */
	private Sheet sheet;
	
	/**
	 * 样式列表
	 */
	private Map<String, CellStyle> styles;
	
	/**
	 * 当前行号
	 */
	private int rownum;
	
	/**
	 * 注解列表（Object[]{ ExcelField, Field/Method }）
	 */
	List<Object[]> annotationList = Lists.newArrayList();
	
	/**
	 * 构造函数
	 * @param title 表格标题，传“空值”，表示无标题
	 * @param cls 实体对象，通过annotation.ExportField获取标题
	 */
	public ExportExcel(String title, Class<?> cls){
		this(title, cls, 1);
	}
	
	/**
	 * 构造函数
	 * @param title 表格标题，传“空值”，表示无标题
	 * @param cls 实体对象，通过annotation.ExportField获取标题
	 * @param type 导出类型（1:导出数据；2：导出模板）
	 * @param groups 导入分组
	 */
	public ExportExcel(String title, Class<?> cls, int type, int... groups){
		// Get annotation field 
		Field[] fs = cls.getDeclaredFields();
		for (Field f : fs){
			ExcelField ef = f.getAnnotation(ExcelField.class);
			if (ef != null && (ef.type()==0 || ef.type()==type)){
				if (groups!=null && groups.length>0){
					boolean inGroup = false;
					for (int g : groups){
						if (inGroup){
							break;
						}
						for (int efg : ef.groups()){
							if (g == efg){
								inGroup = true;
								annotationList.add(new Object[]{ef, f});
								break;
							}
						}
					}
				}else{
					annotationList.add(new Object[]{ef, f});
				}
			}
		}
		// Get annotation method
		Method[] ms = cls.getDeclaredMethods();
		for (Method m : ms){
			ExcelField ef = m.getAnnotation(ExcelField.class);
			if (ef != null && (ef.type()==0 || ef.type()==type)){
				if (groups!=null && groups.length>0){
					boolean inGroup = false;
					for (int g : groups){
						if (inGroup){
							break;
						}
						for (int efg : ef.groups()){
							if (g == efg){
								inGroup = true;
								annotationList.add(new Object[]{ef, m});
								break;
							}
						}
					}
				}else{
					annotationList.add(new Object[]{ef, m});
				}
			}
		}
		// Field sorting
		Collections.sort(annotationList, new Comparator<Object[]>() {
			public int compare(Object[] o1, Object[] o2) {
				return new Integer(((ExcelField)o1[0]).sort()).compareTo(
						new Integer(((ExcelField)o2[0]).sort()));
			};
		});
		// Initialize
        if(type==2){
            initializeTemplate(title, annotationList);
            return;
        }
		List<String> headerList = Lists.newArrayList();
		for (Object[] os : annotationList){
			String t = ((ExcelField)os[0]).title();
			// 如果是导出，则去掉注释
			if (type==1){
				String[] ss = StringUtils.split(t, "**", 2);
				if (ss.length==2){
					t = ss[0];
				}
			}
			headerList.add(t);
		}
		initialize(title, headerList);
	}

    private void initializeTemplate(String title, List<Object[]> annotationList) {
        this.wb = new SXSSFWorkbook(500);
        this.sheet = wb.createSheet("Export");
        this.styles = createStyles(wb);

        // Create title
        if (StringUtils.isNotBlank(title)){
            Row titleRow = sheet.createRow(rownum++);
            titleRow.setHeightInPoints(30);
            Cell titleCell = titleRow.createCell(0);
            titleCell.setCellStyle(styles.get("title"));
            titleCell.setCellValue(title);
            sheet.addMergedRegion(new CellRangeAddress(titleRow.getRowNum(),
                    titleRow.getRowNum(), titleRow.getRowNum(), annotationList.size()-1));
        }
        // Create header
        if (annotationList == null){
            throw new RuntimeException("headerList not null!");
        }

        List<String> headerList = Lists.newArrayList();
        for (Object[] os : annotationList){
            String t = ((ExcelField)os[0]).title();
            // 如果是导出，则去掉注释
            String[] ss = StringUtils.split(t, "**", 2);
            if (ss.length==2){
                t = ss[0];
            }
            headerList.add(t);
        }

        Row headerRow = sheet.createRow(rownum++);
        headerRow.setHeightInPoints(16);
        for (int i = 0; i < headerList.size(); i++) {
            Cell cell = headerRow.createCell(i);
            cell.setCellStyle(styles.get("header"));
            String[] ss = StringUtils.split(headerList.get(i), "**", 2);
            if (ss.length==2){
                cell.setCellValue(ss[0]);
                Comment comment = this.sheet.createDrawingPatriarch().createCellComment(
                        new XSSFClientAnchor(0, 0, 0, 0, (short) 3, 3, (short) 5, 6));
                comment.setString(new XSSFRichTextString(ss[1]));
                cell.setCellComment(comment);
            }else{
                cell.setCellValue(headerList.get(i));
            }
            sheet.autoSizeColumn(i);
            Object[] objects = annotationList.get(i);
            String dictType = ((ExcelField) objects[0]).dictType();
            if(StringUtils.isNotBlank(dictType)){
                List<Dict> dictList = DictUtils.getDictList(dictType);
                if(dictList!=null&&dictList.size()>0) {
                    String[] arr = new String[dictList.size()];
                    for (int j = 0; j < dictList.size(); j++) {
                        arr[j] = dictList.get(j).getLabel();
                    }
                    sheet.addValidationData(createDataValidation(arr, 2, 5000, i, i));
                    continue;
                }
            }
            String mainDictType = ((ExcelField) objects[0]).mainDictType();
            if(StringUtils.isNotBlank(mainDictType)){
                List<MainDictDetail> dictList = DictUtils.getMainDictList(mainDictType);
                if(dictList!=null&&dictList.size()>0) {
                    String[] arr = new String[dictList.size()];
                    for (int j = 0; j < dictList.size(); j++) {
                        arr[j] = dictList.get(j).getLabel();
                    }
                    sheet.addValidationData(createDataValidation(arr, 2, 5000, i, i));
                    continue;
                }
            }

            String valiName = ((ExcelField) objects[0]).valiName();
            if(StringUtils.isNotBlank(valiName)){
                String[] arr = null;
                if(valiName.equals("WorkJobGrade")){
                    WorkJobGradeService workJobGradeService = SpringContextHolder.getBean(WorkJobGradeService.class);
                    WorkJobGrade workJobGrade = new WorkJobGrade();
                    List<WorkJobGrade> list = workJobGradeService.findList(workJobGrade);
                    arr = new String[list.size()];
                    for (int j = 0; j < list.size(); j++) {
                        arr[j] = list.get(j).getName();
                    }
                }else {
                    List<Dict> dictList = DictUtils.getDictList(valiName);
                    if(dictList!=null&&dictList.size()>0) {
                        arr = new String[dictList.size()];
                        for (int j = 0; j < dictList.size(); j++) {
                            arr[j] = dictList.get(j).getLabel();
                        }
                    }else {
                        List<MainDictDetail> mainDictList = DictUtils.getMainDictList(valiName);
                        arr = new String[mainDictList.size()];
                        for (int j = 0; j < mainDictList.size(); j++) {
                            arr[j] = mainDictList.get(j).getLabel();
                        }
                    }
                }
                if(arr!=null&&arr.length>0) {
                    sheet.addValidationData(createDataValidation(arr, 2, 5000, i, i));
                }
                continue;
            }
        }
        for (int i = 0; i < headerList.size(); i++) {
            int colWidth = sheet.getColumnWidth(i)*2;
            sheet.setColumnWidth(i, colWidth < 3000 ? 3000 : colWidth);
        }
        log.debug("Initialize success.");
    }

    /**
	 * 构造函数
	 * @param title 表格标题，传“空值”，表示无标题
	 * @param cls 实体对象，通过annotation.ExportField获取标题
	 * @param type 导出类型（1:导出数据；2：导出模板）
	 * @param groups 导入分组
	 */
	public ExportExcel(boolean b,String title, Class<?> cls, int type, int... groups){
		// Get annotation field
		Field[] fs = cls.getDeclaredFields();
		for (Field f : fs){
			ExcelField ef = f.getAnnotation(ExcelField.class);
			if (ef != null && (ef.type()==0 || ef.type()==type)){
				if (groups!=null && groups.length>0){
					boolean inGroup = false;
					for (int g : groups){
						if (inGroup){
							break;
						}
						for (int efg : ef.groups()){
							if (g == efg){
								inGroup = true;
								annotationList.add(new Object[]{ef, f});
								break;
							}
						}
					}
				}else{
					annotationList.add(new Object[]{ef, f});
				}
			}
		}
		// Get annotation method
		Method[] ms = cls.getDeclaredMethods();
		for (Method m : ms){
			ExcelField ef = m.getAnnotation(ExcelField.class);
			if (ef != null && (ef.type()==0 || ef.type()==type)){
				if (groups!=null && groups.length>0){
					boolean inGroup = false;
					for (int g : groups){
						if (inGroup){
							break;
						}
						for (int efg : ef.groups()){
							if (g == efg){
								inGroup = true;
								annotationList.add(new Object[]{ef, m});
								break;
							}
						}
					}
				}else{
					annotationList.add(new Object[]{ef, m});
				}
			}
		}
		// Field sorting
		Collections.sort(annotationList, new Comparator<Object[]>() {
			public int compare(Object[] o1, Object[] o2) {
				return new Integer(((ExcelField)o1[0]).sort()).compareTo(
						new Integer(((ExcelField)o2[0]).sort()));
			};
		});
		// Initialize
        if(type==2){
            int size = annotationList.size();
            if (b){
                for (int i = 1; i <= 3; i++) {
                    annotationList.remove(size-i);
                }
            }
            initializeTemplate(title, annotationList);
            return;
        }
		List<String> headerList = Lists.newArrayList();
		if (b){
			if (annotationList.size()>3) {
				for (int i = 0; i < annotationList.size() - 3; i++) {
					Object[] os = annotationList.get(i);
					String t = ((ExcelField) os[0]).title();
					// 如果是导出，则去掉注释
					if (type == 1) {
						String[] ss = StringUtils.split(t, "**", 2);
						if (ss.length == 2) {
							t = ss[0];
						}
					}
					headerList.add(t);
				}
			}else {
				for (Object[] os : annotationList){
					String t = ((ExcelField)os[0]).title();
					// 如果是导出，则去掉注释
					if (type==1){
						String[] ss = StringUtils.split(t, "**", 2);
						if (ss.length==2){
							t = ss[0];
						}
					}
					headerList.add(t);
				}
			}
		}else {
			for (Object[] os : annotationList){
				String t = ((ExcelField)os[0]).title();
				// 如果是导出，则去掉注释
				if (type==1){
					String[] ss = StringUtils.split(t, "**", 2);
					if (ss.length==2){
						t = ss[0];
					}
				}
				headerList.add(t);
			}
		}
		initialize(title, headerList);
	}

	/**
	 * 构造函数
	 * @param title 表格标题，传“空值”，表示无标题
	 * @param headers 表头数组
	 */
	public ExportExcel(String title, String[] headers) {
		initialize(title, Lists.newArrayList(headers));
	}
	
	/**
	 * 构造函数
	 * @param title 表格标题，传“空值”，表示无标题
	 * @param headerList 表头列表
	 */
	public ExportExcel(String title, List<String> headerList) {
		initialize(title, headerList);
	}
	
	/**
	 * 初始化函数
	 * @param title 表格标题，传“空值”，表示无标题
	 * @param headerList 表头列表
	 */
	private void initialize(String title, List<String> headerList) {
		this.wb = new SXSSFWorkbook(500);
		this.sheet = wb.createSheet("Export");
		this.styles = createStyles(wb);

		// Create title
		if (StringUtils.isNotBlank(title)){
			Row titleRow = sheet.createRow(rownum++);
			titleRow.setHeightInPoints(30);
			Cell titleCell = titleRow.createCell(0);
			titleCell.setCellStyle(styles.get("title"));
			titleCell.setCellValue(title);
			sheet.addMergedRegion(new CellRangeAddress(titleRow.getRowNum(),
					titleRow.getRowNum(), titleRow.getRowNum(), headerList.size()-1));
		}
		// Create header
		if (headerList == null){
			throw new RuntimeException("headerList not null!");
		}
		Row headerRow = sheet.createRow(rownum++);
		headerRow.setHeightInPoints(16);
		for (int i = 0; i < headerList.size(); i++) {
			Cell cell = headerRow.createCell(i);
			cell.setCellStyle(styles.get("header"));
			String[] ss = StringUtils.split(headerList.get(i), "**", 2);
			if (ss.length==2){
				cell.setCellValue(ss[0]);
				Comment comment = this.sheet.createDrawingPatriarch().createCellComment(
						new XSSFClientAnchor(0, 0, 0, 0, (short) 3, 3, (short) 5, 6));
				comment.setString(new XSSFRichTextString(ss[1]));
				cell.setCellComment(comment);
			}else{
				cell.setCellValue(headerList.get(i));
			}
			sheet.autoSizeColumn(i);
		}
		for (int i = 0; i < headerList.size(); i++) {  
			int colWidth = sheet.getColumnWidth(i)*2;
	        sheet.setColumnWidth(i, colWidth < 3000 ? 3000 : colWidth);  
		}
		log.debug("Initialize success.");
	}

    private DataValidation createDataValidation(String[] textList, int firstRow, int endRow, int firstCol, int endCol) {

        DataValidationHelper helper = sheet.getDataValidationHelper();
        //加载下拉列表内容
        DataValidationConstraint constraint = helper.createExplicitListConstraint(textList);
        //DVConstraint constraint = new DVConstraint();
        constraint.setExplicitListValues(textList);

        //设置数据有效性加载在哪个单元格上。四个参数分别是：起始行、终止行、起始列、终止列
        CellRangeAddressList regions = new CellRangeAddressList(firstRow, endRow, firstCol, endCol);

        //数据有效性对象
        DataValidation data_validation = helper.createValidation(constraint, regions);
        //DataValidation data_validation = new DataValidation(regions, constraint);
        data_validation.createErrorBox("error","数据不符合规范");

        return data_validation;
    }

	/**
	 * 创建表格样式
	 * @param wb 工作薄对象
	 * @return 样式列表
	 */
	private Map<String, CellStyle> createStyles(Workbook wb) {
		Map<String, CellStyle> styles = new HashMap<String, CellStyle>();
		
		CellStyle style = wb.createCellStyle();
		style.setAlignment(CellStyle.ALIGN_CENTER);
		style.setVerticalAlignment(CellStyle.VERTICAL_CENTER);
		style.setBorderRight(CellStyle.BORDER_THIN);
		style.setRightBorderColor(IndexedColors.GREY_50_PERCENT.getIndex());
		style.setBorderLeft(CellStyle.BORDER_THIN);
		style.setLeftBorderColor(IndexedColors.GREY_50_PERCENT.getIndex());
		style.setBorderTop(CellStyle.BORDER_THIN);
		style.setTopBorderColor(IndexedColors.GREY_50_PERCENT.getIndex());
		style.setBorderBottom(CellStyle.BORDER_THIN);
		style.setBottomBorderColor(IndexedColors.GREY_50_PERCENT.getIndex());
		Font titleFont = wb.createFont();
		titleFont.setFontName("Arial");
		titleFont.setFontHeightInPoints((short) 16);
		titleFont.setBoldweight(Font.BOLDWEIGHT_BOLD);
		style.setFont(titleFont);
		styles.put("title", style);

		style = wb.createCellStyle();
		style.setVerticalAlignment(CellStyle.VERTICAL_CENTER);
		style.setBorderRight(CellStyle.BORDER_THIN);
		style.setRightBorderColor(IndexedColors.GREY_50_PERCENT.getIndex());
		style.setBorderLeft(CellStyle.BORDER_THIN);
		style.setLeftBorderColor(IndexedColors.GREY_50_PERCENT.getIndex());
		style.setBorderTop(CellStyle.BORDER_THIN);
		style.setTopBorderColor(IndexedColors.GREY_50_PERCENT.getIndex());
		style.setBorderBottom(CellStyle.BORDER_THIN);
		style.setBottomBorderColor(IndexedColors.GREY_50_PERCENT.getIndex());
		Font dataFont = wb.createFont();
		dataFont.setFontName("Arial");
		dataFont.setFontHeightInPoints((short) 11);
		style.setFont(dataFont);
		styles.put("data", style);
		
		style = wb.createCellStyle();
		style.cloneStyleFrom(styles.get("data"));
		style.setAlignment(CellStyle.ALIGN_LEFT);
		styles.put("data1", style);

		style = wb.createCellStyle();
		style.cloneStyleFrom(styles.get("data"));
		style.setAlignment(CellStyle.ALIGN_CENTER);
		styles.put("data2", style);

		style = wb.createCellStyle();
		style.cloneStyleFrom(styles.get("data"));
		style.setAlignment(CellStyle.ALIGN_RIGHT);
		styles.put("data3", style);
		
		style = wb.createCellStyle();
		style.cloneStyleFrom(styles.get("data"));
//		style.setWrapText(true);
		style.setAlignment(CellStyle.ALIGN_CENTER);
		style.setFillForegroundColor(IndexedColors.GREY_50_PERCENT.getIndex());
		style.setFillPattern(CellStyle.SOLID_FOREGROUND);
		Font headerFont = wb.createFont();
		headerFont.setFontName("Arial");
		headerFont.setFontHeightInPoints((short) 10);
		headerFont.setBoldweight(Font.BOLDWEIGHT_BOLD);
		headerFont.setColor(IndexedColors.WHITE.getIndex());
		style.setFont(headerFont);
		styles.put("header", style);
		
		return styles;
	}

	/**
	 * 添加一行
	 * @return 行对象
	 */
	public Row addRow(){
		return sheet.createRow(rownum++);
	}
	

	/**
	 * 添加一个单元格
	 * @param row 添加的行
	 * @param column 添加列号
	 * @param val 添加值
	 * @return 单元格对象
	 */
	public Cell addCell(Row row, int column, Object val){
		return this.addCell(row, column, val, 0, Class.class);
	}
	
	/**
	 * 添加一个单元格
	 * @param row 添加的行
	 * @param column 添加列号
	 * @param val 添加值
	 * @param align 对齐方式（1：靠左；2：居中；3：靠右）
	 * @return 单元格对象
	 */
	public Cell addCell(Row row, int column, Object val, int align, Class<?> fieldType){
		Cell cell = row.createCell(column);
		CellStyle style = styles.get("data"+(align>=1&&align<=3?align:""));
		try {
			if (val == null){
				cell.setCellValue("");
			} else if (val instanceof String) {
				cell.setCellValue((String) val);
			} else if (val instanceof Integer) {
				cell.setCellValue((Integer) val);
			} else if (val instanceof Long) {
				cell.setCellValue((Long) val);
			} else if (val instanceof Double) {
				cell.setCellValue((Double) val);
			} else if (val instanceof Float) {
				cell.setCellValue((Float) val);
			} else if (val instanceof Date) {
				DataFormat format = wb.createDataFormat();
	            style.setDataFormat(format.getFormat("yyyy-MM-dd"));
				cell.setCellValue((Date) val);
			} else {
				if (fieldType != Class.class){
					cell.setCellValue((String)fieldType.getMethod("setValue", Object.class).invoke(null, val));
				}else{
					cell.setCellValue((String)Class.forName(this.getClass().getName().replaceAll(this.getClass().getSimpleName(), 
						"fieldtype."+val.getClass().getSimpleName()+"Type")).getMethod("setValue", Object.class).invoke(null, val));
				}
			}
		} catch (Exception ex) {
			log.info("Set cell value ["+row.getRowNum()+","+column+"] error: " + ex.toString());
			cell.setCellValue(val.toString());
		}
		cell.setCellStyle(style);
		return cell;
	}

	/**
	 * 添加数据（通过annotation.ExportField添加数据）
	 * @return list 数据列表
	 */
	public <E> ExportExcel setDataList(List<E> list){
		for (E e : list){
			int colunm = 0;
			Row row = this.addRow();
			StringBuilder sb = new StringBuilder();
			for (Object[] os : annotationList){
				ExcelField ef = (ExcelField)os[0];
				Object val = null;
				// Get entity value
				try{
					if (StringUtils.isNotBlank(ef.value())){
						val = Reflections.invokeGetter(e, ef.value());
					}else{
						if (os[1] instanceof Field){
							val = Reflections.invokeGetter(e, ((Field)os[1]).getName());
						}else if (os[1] instanceof Method){
							val = Reflections.invokeMethod(e, ((Method)os[1]).getName(), new Class[] {}, new Object[] {});
						}
					}
					// If is dict, get dict label
					if (StringUtils.isNotBlank(ef.dictType())){
						val = DictUtils.getDictLabel(val==null?"":val.toString(), ef.dictType(), "");
					}else if (StringUtils.isNotBlank(ef.mainDictType())){
						val = DictUtils.getMainDictLabel(val==null?"":val.toString(), ef.mainDictType(), "");
					}
				}catch(Exception ex) {
					// Failure to ignore
					log.info(ex.toString());
					val = "";
				}
				this.addCell(row, colunm++, val, ef.align(), ef.fieldType());
				sb.append(val + ", ");
			}
			log.debug("Write success: ["+row.getRowNum()+"] "+sb.toString());
		}
		return this;
	}

	/**
	 * 添加数据（通过annotation.ExportField添加数据）
	 * @return list 数据列表
	 */
	public <E> ExportExcel setDataList(boolean b,List<E> list){
		if (b){
			for (E e : list) {
				int colunm = 0;
				Row row = this.addRow();
				StringBuilder sb = new StringBuilder();
				if (annotationList.size()>=3){
					for (int i = 0 ;i < annotationList.size()-3 ; i++){
						Object[] os = annotationList.get(i);
						ExcelField ef = (ExcelField) os[0];
						Object val = null;
						// Get entity value
						try {
							if (StringUtils.isNotBlank(ef.value())) {
								val = Reflections.invokeGetter(e, ef.value());
							} else {
								if (os[1] instanceof Field) {
									val = Reflections.invokeGetter(e, ((Field) os[1]).getName());
								} else if (os[1] instanceof Method) {
									val = Reflections.invokeMethod(e, ((Method) os[1]).getName(), new Class[]{}, new Object[]{});
								}
							}
							// If is dict, get dict label
							if (StringUtils.isNotBlank(ef.dictType())) {
								val = DictUtils.getDictLabel(val == null ? "" : val.toString(), ef.dictType(), "");
							}else if (StringUtils.isNotBlank(ef.mainDictType())){
								val = DictUtils.getMainDictLabel(val==null?"":val.toString(), ef.mainDictType(), "");
							}
						} catch (Exception ex) {
							// Failure to ignore
							log.info(ex.toString());
							val = "";
						}
						this.addCell(row, colunm++, val, ef.align(), ef.fieldType());
						sb.append(val + ", ");
					}
					log.debug("Write success: [" + row.getRowNum() + "] " + sb.toString());
				}else {
					for (Object[] os : annotationList) {
						ExcelField ef = (ExcelField) os[0];
						Object val = null;
						// Get entity value
						try {
							if (StringUtils.isNotBlank(ef.value())) {
								val = Reflections.invokeGetter(e, ef.value());
							} else {
								if (os[1] instanceof Field) {
									val = Reflections.invokeGetter(e, ((Field) os[1]).getName());
								} else if (os[1] instanceof Method) {
									val = Reflections.invokeMethod(e, ((Method) os[1]).getName(), new Class[]{}, new Object[]{});
								}
							}
							// If is dict, get dict label
							if (StringUtils.isNotBlank(ef.dictType())) {
								val = DictUtils.getDictLabel(val == null ? "" : val.toString(), ef.dictType(), "");
							}else if (StringUtils.isNotBlank(ef.mainDictType())){
								val = DictUtils.getMainDictLabel(val==null?"":val.toString(), ef.mainDictType(), "");
							}
						} catch (Exception ex) {
							// Failure to ignore
							log.info(ex.toString());
							val = "";
						}
						this.addCell(row, colunm++, val, ef.align(), ef.fieldType());
						sb.append(val + ", ");
					}
					log.debug("Write success: [" + row.getRowNum() + "] " + sb.toString());
				}
			}
		}else {
			for (E e : list) {
				int colunm = 0;
				Row row = this.addRow();
				StringBuilder sb = new StringBuilder();
				for (Object[] os : annotationList) {
					ExcelField ef = (ExcelField) os[0];
					Object val = null;
					// Get entity value
					try {
						if (StringUtils.isNotBlank(ef.value())) {
							val = Reflections.invokeGetter(e, ef.value());
						} else {
							if (os[1] instanceof Field) {
								val = Reflections.invokeGetter(e, ((Field) os[1]).getName());
							} else if (os[1] instanceof Method) {
								val = Reflections.invokeMethod(e, ((Method) os[1]).getName(), new Class[]{}, new Object[]{});
							}
						}
						// If is dict, get dict label
						if (StringUtils.isNotBlank(ef.dictType())) {
							val = DictUtils.getDictLabel(val == null ? "" : val.toString(), ef.dictType(), "");
						}else if (StringUtils.isNotBlank(ef.mainDictType())){
							val = DictUtils.getMainDictLabel(val==null?"":val.toString(), ef.mainDictType(), "");
						}
					} catch (Exception ex) {
						// Failure to ignore
						log.info(ex.toString());
						val = "";
					}
					this.addCell(row, colunm++, val, ef.align(), ef.fieldType());
					sb.append(val + ", ");
				}
				log.debug("Write success: [" + row.getRowNum() + "] " + sb.toString());
			}
		}
		return this;
	}
	
	/**
	 * 输出数据流
	 * @param os 输出数据流
	 */
	public ExportExcel write(OutputStream os) throws IOException{
		wb.write(os);
		return this;
	}
	
	/**
	 * 输出到客户端
	 * @param fileName 输出文件名
	 */
	public ExportExcel write(HttpServletResponse response, String fileName) throws IOException{
		response.reset();
        response.setContentType("application/octet-stream; charset=utf-8");
		response.setHeader("Content-Disposition", "attachment; filename="+ new String(fileName.getBytes("utf-8"),"ISO-8859-1"));
		//response.setHeader("Content-Disposition", "attachment; filename="+Encodes.urlEncode(fileName));
		write(response.getOutputStream());
		return this;
	}

	/**
	 * 输出到客户端
	 * @param fileName 输出文件名
	 */
	public ExportExcel write(HttpServletResponse response, String fileName,String agent) throws IOException{
		response.reset();
        response.setContentType("application/octet-stream; charset=utf-8");
		if(agent != null && agent.toLowerCase().indexOf("firefox") > 0){
			response.setHeader("Content-Disposition", "attachment; filename="+ new String(fileName.getBytes("utf-8"),"ISO-8859-1"));
		}else {
			response.setHeader("Content-Disposition", "attachment; filename="+Encodes.urlEncode(fileName));
		}
		write(response.getOutputStream());
		return this;
	}

	/**
	 * 输出到文件
	 * @param name 输出文件名
	 */
	public ExportExcel writeFile(String name) throws FileNotFoundException, IOException{
		FileOutputStream os = new FileOutputStream(name);
		this.write(os);
		return this;
	}
	
	/**
	 * 清理临时文件
	 */
	public ExportExcel dispose(){
		wb.dispose();
		return this;
	}
	
//	/**
//	 * 导出测试
//	 */
//	public static void main(String[] args) throws Throwable {
//		
//		List<String> headerList = Lists.newArrayList();
//		for (int i = 1; i <= 10; i++) {
//			headerList.add("表头"+i);
//		}
//		
//		List<String> dataRowList = Lists.newArrayList();
//		for (int i = 1; i <= headerList.size(); i++) {
//			dataRowList.add("数据"+i);
//		}
//		
//		List<List<String>> dataList = Lists.newArrayList();
//		for (int i = 1; i <=1000000; i++) {
//			dataList.add(dataRowList);
//		}
//
//		ExportExcel ee = new ExportExcel("表格标题", headerList);
//		
//		for (int i = 0; i < dataList.size(); i++) {
//			Row row = ee.addRow();
//			for (int j = 0; j < dataList.get(i).size(); j++) {
//				ee.addCell(row, j, dataList.get(i).get(j));
//			}
//		}
//		
//		ee.writeFile("target/export.xlsx");
//
//		ee.dispose();
//		
//		log.debug("Export success.");
//		
//	}

	/**
	 * 动态生成案例模板
	 * @param title sheetName
	 * @param indicesList 指标项
	 * @param headerList 表头内容
	 * @param type 是否单位赋值
	 */
	private void exampleTemplate(SXSSFWorkbook wb, String title, List<WorkExampleIndices> indicesList, List<String> headerList,String type) {
		    this.sheet = wb.createSheet(title);
			this.styles = createStyles(wb);
			this.rownum=0;
			// Create title
			if (StringUtils.isNotBlank(title)){
				Row titleRow = sheet.createRow(rownum++);
				titleRow.setHeightInPoints(30);
				Cell titleCell = titleRow.createCell(0);
				titleCell.setCellStyle(styles.get("title"));
				titleCell.setCellValue(title);
				sheet.addMergedRegion(new CellRangeAddress(titleRow.getRowNum(),
						titleRow.getRowNum(), titleRow.getRowNum(), headerList.size()-1));
			}
			// Create header
			if (headerList == null){
				throw new RuntimeException("headerList not null!");
			}
			Row headerRow = sheet.createRow(rownum++);
			headerRow.setHeightInPoints(20);
			for (int i = 0; i < headerList.size(); i++) {
				Cell cell = headerRow.createCell(i);
				cell.setCellStyle(styles.get("header"));
				String[] ss = StringUtils.split(headerList.get(i), "**", 2);
				if (ss.length==2){
					cell.setCellValue(ss[0]);
					Comment comment = this.sheet.createDrawingPatriarch().createCellComment(
							new XSSFClientAnchor(0, 0, 0, 0, (short) 3, 3, (short) 5, 6));
					comment.setString(new XSSFRichTextString(ss[1]));
					cell.setCellComment(comment);
				}else{
					cell.setCellValue(headerList.get(i));
				}
				sheet.autoSizeColumn(i);
			}
			for(WorkExampleIndices workExampleIndices:indicesList){
				Row contentRow=sheet.createRow(rownum++);
				contentRow.setHeightInPoints(20);
				for (int i = 0; i < headerList.size(); i++) {
					Cell cell = contentRow.createCell(i);
					cell.setCellStyle(styles.get("data"));
					if(i==0){
						cell.setCellValue(workExampleIndices.getName());
					}
					if(i==1){
						if("2".equals(type)){
							cell.setCellValue(DictUtils.getMainDictLabel(workExampleIndices.getItemUnit(),"unit_type",""));
						}
					}
					sheet.autoSizeColumn(i);
				}

			}
			for (int i = 0; i < headerList.size(); i++) {
				int colWidth = sheet.getColumnWidth(i)*2;
				sheet.setColumnWidth(i, colWidth < 3000 ? 3000 : colWidth);
			}

		log.debug("Initialize success.");
	}
	/**
	 *
	 * @param title sheetName
	 * @param indicesList 指标项
	 * @param headerList 表头内容
	 * @param type 是否单位赋值
	 */
	public void createSheet(String title, List<WorkExampleIndices> indicesList, List<String> headerList,String type) {
		exampleTemplate(wb,title,indicesList,headerList,type);

	}
	public ExportExcel(int i){
		this.wb=new SXSSFWorkbook(i);
		this.sheet = wb.createSheet("工程概况");
		this.styles = createStyles(wb);
		Row titleRow = sheet.createRow(rownum++);
		titleRow.setHeightInPoints(40);
		Cell titleCell = titleRow.createCell(0);
		titleCell.setCellStyle(styles.get("title"));
		titleCell.setCellValue("工程概况");
		sheet.addMergedRegion(new CellRangeAddress(titleRow.getRowNum(),
				titleRow.getRowNum(), titleRow.getRowNum(), 3));
		sheet.setColumnWidth(0,6000);
		sheet.setColumnWidth(1,6000);
		sheet.setColumnWidth(2,6000);
		sheet.setColumnWidth(3,6000);
		createRows(sheet,"工程名称","工程类型");
		createRows(sheet,"工程地点","建设规模(m2)");
		createRows(sheet,"结构形式","基础类型");
		createRows(sheet,"规模(户数、床位数等)","结构安全等级");
		createRows(sheet,"编制阶段","投资类型");
		createRows(sheet,"计价类型","计价依据");
		createRows(sheet,"招标基准信息价","计税模式");
		createRows(sheet,"开工日期","竣工日期");
		createRows(sheet,"地上建筑面积","地下建筑面积");
		createRows(sheet,"人防建筑面积","车库建筑面积");
		createRows(sheet,"地上层数","地下层数");
		createRows(sheet,"地上层高","地下层高");
		createRows(sheet,"总层数","檐高");
		createRows(sheet,"内墙面装修","外墙面装修");
		createRows(sheet,"天棚装修","楼地面装修");
		createRows(sheet,"门","窗");
		createRows(sheet,"说明","");

	}
	public void createRows(Sheet sheet,String str1,String str2){
		Row contentRow = sheet.createRow(rownum++);
		contentRow.setHeightInPoints(30);
		for(int i=0;i<4;i++){
			Cell cell = contentRow.createCell(i);
			cell.setCellStyle(styles.get("data"));
			if(i==0){
				cell.setCellValue(str1);
			}else if(i==2){
				cell.setCellValue(str2);
			}
		}
	}
}
