/**
 * Copyright &copy; 2015-2020 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.modules.sys.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.ConstraintViolationException;

import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.google.common.collect.Lists;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.MyBeanUtils;
import com.jeeplus.common.config.Global;
import com.jeeplus.common.persistence.Page;
import com.jeeplus.common.web.BaseController;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.modules.sys.entity.Suggestion;
import com.jeeplus.modules.sys.service.SuggestionService;

/**
 * 意见反馈Controller
 * @author 朱明月
 * @version 2017-04-21
 */
@Controller
@RequestMapping(value = "${adminPath}/sys/suggestion")
public class SuggestionController extends BaseController {

	@Autowired
	private SuggestionService suggestionService;
	
	@ModelAttribute
	public Suggestion get(@RequestParam(required=false) String id) {
		Suggestion entity = null;
		if (StringUtils.isNotBlank(id)){
			entity = suggestionService.get(id);
		}
		if (entity == null){
			entity = new Suggestion();
		}
		return entity;
	}
	
	/**
	 * 意见反馈列表页面
	 */
	@RequiresPermissions("sys:suggestion:list")
	@RequestMapping(value = {"list", ""})
	public String list(Suggestion suggestion, HttpServletRequest request, HttpServletResponse response, Model model) {
		Page<Suggestion> page = suggestionService.findPage(new Page<Suggestion>(request, response), suggestion); 
		model.addAttribute("page", page);
		return "modules/sys/suggestionList";
	}

	/**
	 * 查看，增加，编辑意见反馈表单页面
	 */
	@RequiresPermissions(value={"sys:suggestion:view","sys:suggestion:add","sys:suggestion:edit"},logical=Logical.OR)
	@RequestMapping(value = "form")
	public String form(Suggestion suggestion, Model model, HttpServletRequest request) {
		model.addAttribute("suggestion", suggestion);
		String view = request.getParameter("view");
		if (!StringUtils.isBlank(view)) {
			return "modules/sys/suggestionView";
		}
		return "modules/sys/suggestionForm";
	}

	/**
	 * 保存意见反馈
	 */
	@RequiresPermissions(value={"sys:suggestion:add","sys:suggestion:edit"},logical=Logical.OR)
	@RequestMapping(value = "save")
	public String save(Suggestion suggestion, Model model, RedirectAttributes redirectAttributes, HttpServletRequest request) throws Exception{
		if (!beanValidator(model, suggestion)){
			return form(suggestion, model,request);
		}
		if(!suggestion.getIsNewRecord()){//编辑表单保存
			Suggestion t = suggestionService.get(suggestion.getId());//从数据库取出记录的值
			MyBeanUtils.copyBeanNotNull2Bean(suggestion, t);//将编辑表单中的非NULL值覆盖数据库记录中的值
			suggestionService.save(t);//保存
		}else{//新增表单保存
			suggestionService.save(suggestion);//保存
		}
		addMessage(redirectAttributes, "保存意见反馈成功");
		return "redirect:"+Global.getAdminPath()+"/sys/suggestion/?repage";
	}
	
	/**
	 * 删除意见反馈
	 */
	@RequiresPermissions("sys:suggestion:del")
	@RequestMapping(value = "delete")
	public String delete(Suggestion suggestion, RedirectAttributes redirectAttributes) {
		suggestionService.delete(suggestion);
		addMessage(redirectAttributes, "删除意见反馈成功");
		return "redirect:"+Global.getAdminPath()+"/sys/suggestion/?repage";
	}
	
	/**
	 * 批量删除意见反馈
	 */
	@RequiresPermissions("sys:suggestion:del")
	@RequestMapping(value = "deleteAll")
	public String deleteAll(String ids, RedirectAttributes redirectAttributes) {
		String idArray[] =ids.split(",");
		for(String id : idArray){
			suggestionService.delete(suggestionService.get(id));
		}
		addMessage(redirectAttributes, "删除意见反馈成功");
		return "redirect:"+Global.getAdminPath()+"/sys/suggestion/?repage";
	}
	
	/**
	 * 导出excel文件
	 */
	@RequiresPermissions("sys:suggestion:export")
    @RequestMapping(value = "export", method=RequestMethod.POST)
    public String exportFile(Suggestion suggestion, HttpServletRequest request, HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "意见反馈"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<Suggestion> page = suggestionService.findPage(new Page<Suggestion>(request, response, -1), suggestion);
    		new ExportExcel("意见反馈", Suggestion.class).setDataList(page.getList()).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导出意见反馈记录失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/sys/suggestion/?repage";
    }

	/**
	 * 导入Excel数据

	 */
	@RequiresPermissions("sys:suggestion:import")
    @RequestMapping(value = "import", method=RequestMethod.POST)
    public String importFile(MultipartFile file, RedirectAttributes redirectAttributes) {
		try {
			int successNum = 0;
			int failureNum = 0;
			StringBuilder failureMsg = new StringBuilder();
			ImportExcel ei = new ImportExcel(file, 1, 0);
			List<Suggestion> list = ei.getDataList(Suggestion.class);
			for (Suggestion suggestion : list){
				try{
					suggestionService.save(suggestion);
					successNum++;
				}catch(ConstraintViolationException ex){
					failureNum++;
				}catch (Exception ex) {
					failureNum++;
				}
			}
			if (failureNum>0){
				failureMsg.insert(0, "，失败 "+failureNum+" 条意见反馈记录。");
			}
			addMessage(redirectAttributes, "已成功导入 "+successNum+" 条意见反馈记录"+failureMsg);
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入意见反馈失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/sys/suggestion/?repage";
    }
	
	/**
	 * 下载导入意见反馈数据模板
	 */
	@RequiresPermissions("sys:suggestion:import")
    @RequestMapping(value = "import/template")
    public String importFileTemplate(HttpServletResponse response, RedirectAttributes redirectAttributes) {
		try {
            String fileName = "意见反馈数据导入模板.xlsx";
    		List<Suggestion> list = Lists.newArrayList(); 
    		new ExportExcel("意见反馈数据", Suggestion.class, 1).setDataList(list).write(response, fileName).dispose();
    		return null;
		} catch (Exception e) {
			addMessage(redirectAttributes, "导入模板下载失败！失败信息："+e.getMessage());
		}
		return "redirect:"+Global.getAdminPath()+"/sys/suggestion/?repage";
    }
	
	
	

}